"""Mind v3 context graph storage.

LanceDB-backed graph for storing:
- Decisions (with reasoning, alternatives, context)
- Entities (files, functions, concepts)
- Patterns (preferences, habits, blind spots)
- Policies (rules, guidelines)
- Exceptions (tribal knowledge)
- Precedents (links between decisions)
- Outcomes (what actually happened)
"""
from .store import GraphStore, get_embedding, generate_id, EMBED_DIM

__all__ = [
    "GraphStore",
    "get_embedding",
    "generate_id",
    "EMBED_DIM",
]
